# coding: utf-8

"""
    SpaceBlocks.Permissions.WebServices.Main.Api Management

    This is the management API.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from spaceblocks_permissions_server.models.create_resource_request import CreateResourceRequest
from spaceblocks_permissions_server.models.get_permissions_response import GetPermissionsResponse
from spaceblocks_permissions_server.models.resource import Resource
from spaceblocks_permissions_server.models.resource_members import ResourceMembers
from spaceblocks_permissions_server.models.update_resource_request import UpdateResourceRequest

from spaceblocks_permissions_server.api_client import ApiClient, RequestSerialized
from spaceblocks_permissions_server.api_response import ApiResponse
from spaceblocks_permissions_server.rest import RESTResponseType


class ResourceApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def create_resource(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant whom the resource belongs")],
        resource_type_id: Annotated[StrictStr, Field(description="The key of the permission level")],
        create_resource_request: Annotated[CreateResourceRequest, Field(description="The create resource request")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Resource:
        """Create a new resource.


        :param tenant_id: The ID of the tenant whom the resource belongs (required)
        :type tenant_id: str
        :param resource_type_id: The key of the permission level (required)
        :type resource_type_id: str
        :param create_resource_request: The create resource request (required)
        :type create_resource_request: CreateResourceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            create_resource_request=create_resource_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '201': "Resource",
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_resource_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant whom the resource belongs")],
        resource_type_id: Annotated[StrictStr, Field(description="The key of the permission level")],
        create_resource_request: Annotated[CreateResourceRequest, Field(description="The create resource request")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Resource]:
        """Create a new resource.


        :param tenant_id: The ID of the tenant whom the resource belongs (required)
        :type tenant_id: str
        :param resource_type_id: The key of the permission level (required)
        :type resource_type_id: str
        :param create_resource_request: The create resource request (required)
        :type create_resource_request: CreateResourceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            create_resource_request=create_resource_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '201': "Resource",
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_resource_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant whom the resource belongs")],
        resource_type_id: Annotated[StrictStr, Field(description="The key of the permission level")],
        create_resource_request: Annotated[CreateResourceRequest, Field(description="The create resource request")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create a new resource.


        :param tenant_id: The ID of the tenant whom the resource belongs (required)
        :type tenant_id: str
        :param resource_type_id: The key of the permission level (required)
        :type resource_type_id: str
        :param create_resource_request: The create resource request (required)
        :type create_resource_request: CreateResourceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            create_resource_request=create_resource_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '201': "Resource",
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_resource_serialize(
        self,
        tenant_id,
        resource_type_id,
        create_resource_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_resource_request is not None:
            _body_params = create_resource_request


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'text/plain', 
                        'application/octet-stream', 
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_resource(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant where the resource belongs to")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        id: Annotated[StrictStr, Field(description="The ID of the deleted resource")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete a resource.


        :param tenant_id: The ID of the tenant where the resource belongs to (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param id: The ID of the deleted resource (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '204': None,
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_resource_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant where the resource belongs to")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        id: Annotated[StrictStr, Field(description="The ID of the deleted resource")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete a resource.


        :param tenant_id: The ID of the tenant where the resource belongs to (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param id: The ID of the deleted resource (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '204': None,
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_resource_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant where the resource belongs to")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        id: Annotated[StrictStr, Field(description="The ID of the deleted resource")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete a resource.


        :param tenant_id: The ID of the tenant where the resource belongs to (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param id: The ID of the deleted resource (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '204': None,
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_resource_serialize(
        self,
        tenant_id,
        resource_type_id,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_resource(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant whom the resource belongs")],
        resource_type_id: Annotated[StrictStr, Field(description="The key of the permission level")],
        id: Annotated[StrictStr, Field(description="The ID of the resource")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Resource:
        """Get a resource by id


        :param tenant_id: The ID of the tenant whom the resource belongs (required)
        :type tenant_id: str
        :param resource_type_id: The key of the permission level (required)
        :type resource_type_id: str
        :param id: The ID of the resource (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "Resource",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_resource_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant whom the resource belongs")],
        resource_type_id: Annotated[StrictStr, Field(description="The key of the permission level")],
        id: Annotated[StrictStr, Field(description="The ID of the resource")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Resource]:
        """Get a resource by id


        :param tenant_id: The ID of the tenant whom the resource belongs (required)
        :type tenant_id: str
        :param resource_type_id: The key of the permission level (required)
        :type resource_type_id: str
        :param id: The ID of the resource (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "Resource",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_resource_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant whom the resource belongs")],
        resource_type_id: Annotated[StrictStr, Field(description="The key of the permission level")],
        id: Annotated[StrictStr, Field(description="The ID of the resource")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get a resource by id


        :param tenant_id: The ID of the tenant whom the resource belongs (required)
        :type tenant_id: str
        :param resource_type_id: The key of the permission level (required)
        :type resource_type_id: str
        :param id: The ID of the resource (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "Resource",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_resource_serialize(
        self,
        tenant_id,
        resource_type_id,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_resource_permissions(
        self,
        tenant_id: StrictStr,
        resource_type_id: StrictStr,
        id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetPermissionsResponse:
        """get_resource_permissions


        :param tenant_id: (required)
        :type tenant_id: str
        :param resource_type_id: (required)
        :type resource_type_id: str
        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_resource_permissions_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "GetPermissionsResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_resource_permissions_with_http_info(
        self,
        tenant_id: StrictStr,
        resource_type_id: StrictStr,
        id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetPermissionsResponse]:
        """get_resource_permissions


        :param tenant_id: (required)
        :type tenant_id: str
        :param resource_type_id: (required)
        :type resource_type_id: str
        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_resource_permissions_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "GetPermissionsResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_resource_permissions_without_preload_content(
        self,
        tenant_id: StrictStr,
        resource_type_id: StrictStr,
        id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """get_resource_permissions


        :param tenant_id: (required)
        :type tenant_id: str
        :param resource_type_id: (required)
        :type resource_type_id: str
        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_resource_permissions_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "GetPermissionsResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_resource_permissions_serialize(
        self,
        tenant_id,
        resource_type_id,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}/{id}/permissions',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_resources(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        subject_id: Annotated[Optional[StrictStr], Field(description="The ID of the subject")] = None,
        permission_scope: Annotated[Optional[StrictStr], Field(description="The slug of the resource type on which the permission to check is defined")] = None,
        permission: Annotated[Optional[List[StrictStr]], Field(description="The permission slug to check (can occur multiple times)")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Resource]:
        """Get all resources within the given tenantId where the subject has the requested permissions.


        :param tenant_id: The ID of the tenant (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param subject_id: The ID of the subject
        :type subject_id: str
        :param permission_scope: The slug of the resource type on which the permission to check is defined
        :type permission_scope: str
        :param permission: The permission slug to check (can occur multiple times)
        :type permission: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_resources_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            subject_id=subject_id,
            permission_scope=permission_scope,
            permission=permission,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "List[Resource]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_resources_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        subject_id: Annotated[Optional[StrictStr], Field(description="The ID of the subject")] = None,
        permission_scope: Annotated[Optional[StrictStr], Field(description="The slug of the resource type on which the permission to check is defined")] = None,
        permission: Annotated[Optional[List[StrictStr]], Field(description="The permission slug to check (can occur multiple times)")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Resource]]:
        """Get all resources within the given tenantId where the subject has the requested permissions.


        :param tenant_id: The ID of the tenant (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param subject_id: The ID of the subject
        :type subject_id: str
        :param permission_scope: The slug of the resource type on which the permission to check is defined
        :type permission_scope: str
        :param permission: The permission slug to check (can occur multiple times)
        :type permission: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_resources_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            subject_id=subject_id,
            permission_scope=permission_scope,
            permission=permission,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "List[Resource]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_resources_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        subject_id: Annotated[Optional[StrictStr], Field(description="The ID of the subject")] = None,
        permission_scope: Annotated[Optional[StrictStr], Field(description="The slug of the resource type on which the permission to check is defined")] = None,
        permission: Annotated[Optional[List[StrictStr]], Field(description="The permission slug to check (can occur multiple times)")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all resources within the given tenantId where the subject has the requested permissions.


        :param tenant_id: The ID of the tenant (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param subject_id: The ID of the subject
        :type subject_id: str
        :param permission_scope: The slug of the resource type on which the permission to check is defined
        :type permission_scope: str
        :param permission: The permission slug to check (can occur multiple times)
        :type permission: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_resources_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            subject_id=subject_id,
            permission_scope=permission_scope,
            permission=permission,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "List[Resource]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_resources_serialize(
        self,
        tenant_id,
        resource_type_id,
        subject_id,
        permission_scope,
        permission,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'permission': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        # process the query parameters
        if subject_id is not None:
            
            _query_params.append(('subjectId', subject_id))
            
        if permission_scope is not None:
            
            _query_params.append(('permissionScope', permission_scope))
            
        if permission is not None:
            
            _query_params.append(('permission', permission))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_resources_by_id(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        subject_id: Annotated[Optional[StrictStr], Field(description="The ID of the subject")] = None,
        permission_scope: Annotated[Optional[StrictStr], Field(description="The slug of the resource type on which the permission to check is defined")] = None,
        permission: Annotated[Optional[List[StrictStr]], Field(description="The permission slug to check (can occur multiple times)")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[str]:
        """Get all resources within the given tenantId where the subject has the requested permissions.


        :param tenant_id: The ID of the tenant (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param subject_id: The ID of the subject
        :type subject_id: str
        :param permission_scope: The slug of the resource type on which the permission to check is defined
        :type permission_scope: str
        :param permission: The permission slug to check (can occur multiple times)
        :type permission: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_resources_by_id_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            subject_id=subject_id,
            permission_scope=permission_scope,
            permission=permission,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "List[str]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_resources_by_id_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        subject_id: Annotated[Optional[StrictStr], Field(description="The ID of the subject")] = None,
        permission_scope: Annotated[Optional[StrictStr], Field(description="The slug of the resource type on which the permission to check is defined")] = None,
        permission: Annotated[Optional[List[StrictStr]], Field(description="The permission slug to check (can occur multiple times)")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[str]]:
        """Get all resources within the given tenantId where the subject has the requested permissions.


        :param tenant_id: The ID of the tenant (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param subject_id: The ID of the subject
        :type subject_id: str
        :param permission_scope: The slug of the resource type on which the permission to check is defined
        :type permission_scope: str
        :param permission: The permission slug to check (can occur multiple times)
        :type permission: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_resources_by_id_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            subject_id=subject_id,
            permission_scope=permission_scope,
            permission=permission,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "List[str]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_resources_by_id_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        subject_id: Annotated[Optional[StrictStr], Field(description="The ID of the subject")] = None,
        permission_scope: Annotated[Optional[StrictStr], Field(description="The slug of the resource type on which the permission to check is defined")] = None,
        permission: Annotated[Optional[List[StrictStr]], Field(description="The permission slug to check (can occur multiple times)")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all resources within the given tenantId where the subject has the requested permissions.


        :param tenant_id: The ID of the tenant (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param subject_id: The ID of the subject
        :type subject_id: str
        :param permission_scope: The slug of the resource type on which the permission to check is defined
        :type permission_scope: str
        :param permission: The permission slug to check (can occur multiple times)
        :type permission: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_resources_by_id_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            subject_id=subject_id,
            permission_scope=permission_scope,
            permission=permission,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "List[str]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_resources_by_id_serialize(
        self,
        tenant_id,
        resource_type_id,
        subject_id,
        permission_scope,
        permission,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'permission': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        # process the query parameters
        if subject_id is not None:
            
            _query_params.append(('subjectId', subject_id))
            
        if permission_scope is not None:
            
            _query_params.append(('permissionScope', permission_scope))
            
        if permission is not None:
            
            _query_params.append(('permission', permission))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )


        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}/select/id',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def patch_resource_members(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The tenantId")],
        resource_type_id: Annotated[StrictStr, Field(description="The resourceTypeId")],
        id: Annotated[StrictStr, Field(description="The resourceId")],
        resource_members: Annotated[ResourceMembers, Field(description="the UpsertMemberRequest object")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ResourceMembers:
        """Patch resource members

        Upsert all members of an existing resource on a specific role in a PATCH bulk operation. An existing member  gets their role updated; a new member gets the role assigned on the resource.  This triggers a recalculation of the permission tree.

        :param tenant_id: The tenantId (required)
        :type tenant_id: str
        :param resource_type_id: The resourceTypeId (required)
        :type resource_type_id: str
        :param id: The resourceId (required)
        :type id: str
        :param resource_members: the UpsertMemberRequest object (required)
        :type resource_members: ResourceMembers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_resource_members_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            resource_members=resource_members,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "ResourceMembers",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def patch_resource_members_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The tenantId")],
        resource_type_id: Annotated[StrictStr, Field(description="The resourceTypeId")],
        id: Annotated[StrictStr, Field(description="The resourceId")],
        resource_members: Annotated[ResourceMembers, Field(description="the UpsertMemberRequest object")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ResourceMembers]:
        """Patch resource members

        Upsert all members of an existing resource on a specific role in a PATCH bulk operation. An existing member  gets their role updated; a new member gets the role assigned on the resource.  This triggers a recalculation of the permission tree.

        :param tenant_id: The tenantId (required)
        :type tenant_id: str
        :param resource_type_id: The resourceTypeId (required)
        :type resource_type_id: str
        :param id: The resourceId (required)
        :type id: str
        :param resource_members: the UpsertMemberRequest object (required)
        :type resource_members: ResourceMembers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_resource_members_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            resource_members=resource_members,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "ResourceMembers",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def patch_resource_members_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The tenantId")],
        resource_type_id: Annotated[StrictStr, Field(description="The resourceTypeId")],
        id: Annotated[StrictStr, Field(description="The resourceId")],
        resource_members: Annotated[ResourceMembers, Field(description="the UpsertMemberRequest object")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Patch resource members

        Upsert all members of an existing resource on a specific role in a PATCH bulk operation. An existing member  gets their role updated; a new member gets the role assigned on the resource.  This triggers a recalculation of the permission tree.

        :param tenant_id: The tenantId (required)
        :type tenant_id: str
        :param resource_type_id: The resourceTypeId (required)
        :type resource_type_id: str
        :param id: The resourceId (required)
        :type id: str
        :param resource_members: the UpsertMemberRequest object (required)
        :type resource_members: ResourceMembers
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_resource_members_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            resource_members=resource_members,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '200': "ResourceMembers",
            '404': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _patch_resource_members_serialize(
        self,
        tenant_id,
        resource_type_id,
        id,
        resource_members,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if resource_members is not None:
            _body_params = resource_members


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'text/plain', 
                        'application/octet-stream', 
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}/{id}/members',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_resource(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant where the resource belongs to")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        id: Annotated[StrictStr, Field(description="The ID of the resource")],
        update_resource_request: Annotated[UpdateResourceRequest, Field(description="The UpdateResourceRequest DTO")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Update a resource

        This triggers a recalculation of the permission tree.

        :param tenant_id: The ID of the tenant where the resource belongs to (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param id: The ID of the resource (required)
        :type id: str
        :param update_resource_request: The UpdateResourceRequest DTO (required)
        :type update_resource_request: UpdateResourceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            update_resource_request=update_resource_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '202': None,
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_resource_with_http_info(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant where the resource belongs to")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        id: Annotated[StrictStr, Field(description="The ID of the resource")],
        update_resource_request: Annotated[UpdateResourceRequest, Field(description="The UpdateResourceRequest DTO")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Update a resource

        This triggers a recalculation of the permission tree.

        :param tenant_id: The ID of the tenant where the resource belongs to (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param id: The ID of the resource (required)
        :type id: str
        :param update_resource_request: The UpdateResourceRequest DTO (required)
        :type update_resource_request: UpdateResourceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            update_resource_request=update_resource_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '202': None,
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_resource_without_preload_content(
        self,
        tenant_id: Annotated[StrictStr, Field(description="The ID of the tenant where the resource belongs to")],
        resource_type_id: Annotated[StrictStr, Field(description="The key to the corresponding permission-level of the resource")],
        id: Annotated[StrictStr, Field(description="The ID of the resource")],
        update_resource_request: Annotated[UpdateResourceRequest, Field(description="The UpdateResourceRequest DTO")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update a resource

        This triggers a recalculation of the permission tree.

        :param tenant_id: The ID of the tenant where the resource belongs to (required)
        :type tenant_id: str
        :param resource_type_id: The key to the corresponding permission-level of the resource (required)
        :type resource_type_id: str
        :param id: The ID of the resource (required)
        :type id: str
        :param update_resource_request: The UpdateResourceRequest DTO (required)
        :type update_resource_request: UpdateResourceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_resource_serialize(
            tenant_id=tenant_id,
            resource_type_id=resource_type_id,
            id=id,
            update_resource_request=update_resource_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '401': "ProblemDetails",
            '202': None,
            '400': "ProblemDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_resource_serialize(
        self,
        tenant_id,
        resource_type_id,
        id,
        update_resource_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant_id is not None:
            _path_params['tenantId'] = tenant_id
        if resource_type_id is not None:
            _path_params['resourceTypeId'] = resource_type_id
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_resource_request is not None:
            _body_params = update_resource_request


        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            [
                'text/plain', 
                'application/json', 
                'text/json'
            ]
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'text/plain', 
                        'application/octet-stream', 
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/management/tenants/{tenantId}/resources/{resourceTypeId}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# flake8: noqa

# import apis into api package
from spaceblocks_permissions_server.api.member_group_api import MemberGroupApi
from spaceblocks_permissions_server.api.permission_api import PermissionApi
from spaceblocks_permissions_server.api.resource_api import ResourceApi
from spaceblocks_permissions_server.api.role_api import RoleApi
from spaceblocks_permissions_server.api.tenant_api import TenantApi




from typing import Optional, Union

import urllib3
from pydantic import BaseModel, validate_call

from .api_client import ApiClient
from .configuration import Configuration

__all__ = [
    'ClientAuthenticationOptions',
    'TokenAuthenticationOptions',
    'SpaceBlocksClientOptions',
    'SpaceBlocksClient'
]


class ClientAuthenticationOptions(BaseModel):
    api_key: str
    client_id: str
    client_secret: str
    scopes: Optional[str] = None


class TokenAuthenticationOptions(BaseModel):
    api_key: str
    access_token: str


class SpaceBlocksClientOptions(BaseModel):
    enable_dev_mode: Optional[bool] = False
    custom_cloud_uri_scheme: Optional[str] = None


class SpaceBlocksClient:
    api_client: ApiClient

    @validate_call
    def __init__(
            self,
            permissions_url: str,
            authentication: Union[ClientAuthenticationOptions, TokenAuthenticationOptions],
            options: Optional[SpaceBlocksClientOptions]
    ):
        if isinstance(authentication, TokenAuthenticationOptions):
            access_token = authentication.access_token
        else:
            access_token = self.get_access_token_from_client_credentials(
                authentication.api_key,
                authentication.client_id,
                authentication.client_secret,
                authentication.scopes,
                options.enable_dev_mode if options else None
            )

        self.api_client = ApiClient(
            Configuration(host=permissions_url)
        )

        self.api_client.default_headers['Authorization'] = f'Bearer {access_token}'
        self.api_client.default_headers['apiKey'] = authentication.api_key

    @staticmethod
    def get_access_token_from_client_credentials(
            api_key: str,
            client_id: str,
            client_secret: str,
            scopes: Optional[str] = None,
            enable_dev_mode: bool = False
    ) -> str:
        space_blocks_auth_domain = 'auth.dev.spaceblocks.cloud' if enable_dev_mode else 'auth.spaceblocks.cloud'

        data = {'client_id': client_id, 'client_secret': client_secret}

        if scopes is not None:
            data['scope'] = scopes

        response = urllib3.request(
            'POST',
            f'https://{space_blocks_auth_domain}/token-manager/token',
            json=data,
            headers={'apiKey': api_key}
        )

        return response.json()['access_token']



# coding: utf-8

# flake8: noqa
"""
    SpaceBlocks.Permissions.WebServices.Main.Api Management

    This is the management API.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from spaceblocks_permissions_server.models.create_member_group_request import CreateMemberGroupRequest
from spaceblocks_permissions_server.models.create_resource_parent import CreateResourceParent
from spaceblocks_permissions_server.models.create_resource_request import CreateResourceRequest
from spaceblocks_permissions_server.models.create_role_request import CreateRoleRequest
from spaceblocks_permissions_server.models.create_tenant_request import CreateTenantRequest
from spaceblocks_permissions_server.models.get_permissions_response import GetPermissionsResponse
from spaceblocks_permissions_server.models.member_group import MemberGroup
from spaceblocks_permissions_server.models.problem_details import ProblemDetails
from spaceblocks_permissions_server.models.resource import Resource
from spaceblocks_permissions_server.models.resource_members import ResourceMembers
from spaceblocks_permissions_server.models.resource_parent import ResourceParent
from spaceblocks_permissions_server.models.role import Role
from spaceblocks_permissions_server.models.tenant import Tenant
from spaceblocks_permissions_server.models.update_resource_parent import UpdateResourceParent
from spaceblocks_permissions_server.models.update_resource_request import UpdateResourceRequest
from spaceblocks_permissions_server.models.update_role_request import UpdateRoleRequest
from spaceblocks_permissions_server.models.update_tenant_request import UpdateTenantRequest
